// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "ShaderStructure.hpp"
#include "display/shader/ShaderData.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

ShaderStructure::ShaderStructure(VariableLayoutReflection *reflection) {
  auto typeLayout = reflection->getTypeLayout();
  auto cName = reflection->getName();
  this->name = cName ? std::string(cName) : "";
  this->size = typeLayout->getSize();
  this->alignment = typeLayout->getAlignment();

  switch(typeLayout->getKind()) {
    case TypeReflection::Kind::Struct: {
      this->type = ShaderStructureType::STRUCT;
      auto count = typeLayout->getFieldCount();
      size_t offset = this->start;
      for(auto i = 0; i < count; i++) {
        auto field = typeLayout->getFieldByIndex(i);
        ShaderStructure member(field);
        member.start = offset;
        offset += member.size;
        this->members.push_back(member);
      }
      assertTrue(offset == this->size, "Struct size/offset mismatch?");
      break;
    }

    case TypeReflection::Kind::Array: {
      this->type = ShaderStructureType::ARRAY;
      this->length = typeLayout->getElementCount();
      break;
    }

    case TypeReflection::Kind::Matrix:
    case TypeReflection::Kind::Vector:
    case TypeReflection::Kind::Scalar:
      this->type = ShaderStructureType::VARIABLE;
      break;

    case TypeReflection::Kind::Resource:
      this->type = ShaderStructureType::RESOURCE;
      break;

    default: {
      assertUnreachable("Unknown type layout kind: %d", typeLayout->getKind());
    }
  }
}

std::shared_ptr<ShaderData> ShaderStructure::createData() {
  return std::make_shared<ShaderData>(shared_from_this());
}