// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"
#include "slang.h"
#include "slang-com-ptr.h"

using namespace slang;
using namespace Slang;

namespace Dawn {
  class ShaderData;

  enum class ShaderStructureType {
    STRUCT,
    ARRAY,
    VARIABLE,
    RESOURCE
  };

  struct ShaderStructure :
    public std::enable_shared_from_this<ShaderStructure>
  {
    public:
      std::string name;
      ShaderStructureType type;

      // Shared properties
      size_t start = 0;
      size_t alignment;
      size_t size;

      // Struct properties
      std::vector<struct ShaderStructure> members;

      // Array properties
      size_t length;

      /**
       * Constructs the ShaderStructure object
       * 
       * @param reflection Reflection data to construct the structure from.
       */
      ShaderStructure(VariableLayoutReflection *reflection);

      /**
       * Creates data for a shader that matches this structure.
       * 
       * @param name Name of the member to get.
       * @return The member structure.
       */
      std::shared_ptr<ShaderData> createData();
  };
}